'''
Tests for unstack.app.app.
'''

from unstack.http.data import Request
from unstack.util import Daemon
from tests.fixtures import app

def test_init(app):
	assert isinstance(app, Daemon)
	assert app.headers == {}
	assert all([
		app.config, app.routes, app.server, app.address, app.config['logger']
	])

def test_contains(app):
	assert ('GET', '/') in app
	assert 404 in app
	assert ('NOPE', '/nope') not in app

def test_route(app):
	test = lambda: 'test'
	app.route('TEST', '/')(test)
	assert app.routes[('TEST', '/')] == test

def test_error(app):
	test = lambda: 'test'
	app.error(123)(test)
	assert app.routes[123] == test

def test_apply(app):
	request, error = 'req', 'err'
	app.routes[('TEST', '/empty')]   = lambda: 'test'
	app.routes[('TEST', '/error')]   = lambda error: error
	app.routes[('TEST', '/request')] = lambda request: request
	app.routes[('TEST', '/req_err')] = lambda request, error: (request, error)
	assert app.apply(('TEST', '/empty'), request, error)   == 'test'
	assert app.apply(('TEST', '/error'), request, error)   == 'err'
	assert app.apply(('TEST', '/request'), request, error) == 'req'
	assert app.apply(('TEST', '/req_err'), request, error) == ('req', 'err')

def test_serve(app):
	req = Request('GET / HTTP/1.1\nFoo: Bar', ('', 0))
	app.routes.update({
		('GET', '/'): lambda: 'test',
		404: lambda: '404',
		500: lambda: '500',
	})
	assert app.serve(req).body == 'test'

	del app.routes[('GET', '/')]
	assert app.serve(req).body == '404'

	app.routes[404] = lambda: 1/0
	assert app.serve(req).body == '500'

	app.routes[500] = lambda: 1/0
	assert app.serve(req).body == 'Error page suffered error: division by zero.'

def test_run(app):
	'''
	Passed. This method is just calls to other tested methods.
	'''

	pass
