'''
Tests for unstack.http.data.headers.
'''

from tests.fixtures    import headers
from unstack.http.data import Headers

def test_init(headers):
	assert headers.headers == {'Foo': 'bar', 'Baz': 'quux'}

def test_contains(headers):
	assert 'Foo' in headers
	assert 'Nope' not in headers

def test_getitem(headers):
	assert headers['Foo'] == 'bar'

def test_delitem(headers):
	del headers['Foo']
	assert 'Foo' not in headers

def test_setitem(headers):
	headers['Foo'] = 'test'
	assert headers['Foo'] == 'test'

def test_str(headers):
	assert str(headers) == 'Baz: quux\nFoo: bar'

def test_update(headers):
	headers.update('String: string')
	assert headers['String'] == 'string'

	headers.update(['List: list', 'Tuple: tuple'])
	assert headers['List']  == 'list'
	assert headers['Tuple'] == 'tuple'

	headers.update({'Dict': 'dict'})
	assert headers['Dict'] == 'dict'

	headers.update(Headers('Headers: headers'))
	assert headers['Headers'] == 'headers'

def test_clean_key(headers):
	for foo in ('Foo', '\tfoo\n', '  FOO  '):
		assert headers.clean_key(foo) == 'Foo'

def test_clean_dict(headers):
	dct = headers.clean_dict({'Foo': 'bar', '\tbaz\n': '  quux  '})
	assert dct == {'Foo': 'bar', 'Baz': 'quux'}

def test_parse_string(headers):
	dct = headers.parse_string('Foo: bar\nBaz: quux')
	assert dct == {'Foo': 'bar', 'Baz': 'quux'}
