'''
Tests for unstack.http.server.
'''

import socket
import threading
import time
from tests.fixtures import request, server

def connect(server):
	socket.setdefaulttimeout(1)
	server.bind()
	sock = socket.create_connection(('127.0.0.1', server.config['port']))
	server.accept()
	return sock

def test_init(server):
	assert all([
		server.config, server.func, server.running, server.sock, server.address
	])
	assert server.connections == server.requests == server.responses == {}

def test_clear(server):
	server.connections['foo'] = 'bar'
	server.requests['foo']    = 'bar'
	server.responses['foo']   = 'bar'
	server.clear('foo')
	assert server.connections == server.requests == server.responses == {}

def test_sleep_if_empty(server):
	t = time.time()
	server.sleep_if_empty()
	assert time.time() > t + server.config['wait_time']

	server.connections = {'foo': 'bar'}
	t = time.time()
	server.sleep_if_empty()
	assert time.time() < t + server.config['wait_time']

def test_bind_accept(server):
	assert not server.connections
	sock = connect(server)
	assert server.connections

def test_request(server):
	assert not server.requests
	sock = connect(server)
	sock.send(b'GET /test HTTP/1.1')
	server.request()
	request = list(server.requests.values())[0]
	assert request.string == 'GET /test HTTP/1.1'

def test_process(server):
	assert not server.responses
	server.requests[('127.0.0.1', 8080)] = request()
	server.process()
	response = list(server.responses.values())[0]
	assert response.body == 'body'

def test_respond(server):
	sock = connect(server)
	sock.send(b'GET /test HTTP/1.1')
	server.request()
	server.process()
	server.respond()
	response = sock.recv(4096).decode().splitlines()
	assert response == [
		'HTTP/1.1 200 OK',
		'Connection: close',
		'Content-Type: text/html',
		'Foo: bar',
		'',
		'body',
	]

def test_start_stop(server):
	thread = threading.Thread(target=server.start)
	thread.start()
	sock = socket.create_connection(('127.0.0.1', server.config['port']))
	sock.send(b'GET /test HTTP/1.1')
	assert '200 OK' in sock.recv(4096).decode()
	server.stop()
	thread.join()
	assert not server.running
	assert server.connections == server.requests == server.responses == {}
