'''
Tests for unstack.http.socket.
'''

import socket
import time
from tests.fixtures import sock1, sock2
from unstack.http import Socket
from unstack.http.data import Response
from unstack.http.socket import if_can

class MockSocket:
	def can(self, mode): return bool(mode)
	@if_can(True)
	def true(self):  return 'test'
	@if_can(False)
	def false(self): return 'test'

def test_if_can():
	sock = MockSocket()
	assert sock.true()
	assert not sock.false()

def test_init():
	for sock in (
		Socket(sock=socket.socket(), address=('test', 123)),
		Socket(host='test', port=123),
		Socket(config={'host': 'test', 'port': 123}),
	):
		assert sock.sock and sock.address == ('test', 123)
		assert sock.host == 'test' and sock.port == 123

def test_str(sock1):
	assert str(sock1) == '127.0.0.1:{}'.format(sock1.port)

def test_basic_connectivity(sock1, sock2):
	# bind, connect, accept
	sock1.bind()
	sock2.connect()
	time.sleep(0.1)
	conn = sock1.accept()
	assert conn
	# can, send, recv
	assert sock2.can('send')
	sock2.send('test')
	assert conn.can('recv')
	assert conn.recv(4096) == 'test'
	assert not conn.can('recv')
	# recv_request, send_response
	sock2.send('GET /test HTTP/1.1')
	request = conn.recv_request(4096)
	assert request.string == 'GET /test HTTP/1.1'
	conn.send_response(Response(request, 200, 'test', {}))
	response = sock2.recv(4096)
	assert 'test' in response
	# close
	for sock in (sock1, sock2, conn):
		sock.close()
		assert not sock.can('recv') and not sock.can('send')
