'''
Log class.
'''

import os
import sys
import time

class Logger:
	'''
	A simple event logger.
	'''

	iso_8601 = '%Y-%m-%d %H:%M:%S'

	def __init__(self, config):
		'''
		Initialise a new Log object.
		'''

		self.config = config

	def __call__(self, text, *args):
		'''
		Log a line of formatted text.
		'''

		text = text.format(*args)
		print('[{}] {}'.format(self.get_timestamp(), text))

	def get_timestamp(self):
		'''
		Return a UTC timestamp of the current time in ISO 8601 format.
		'''

		return time.strftime(self.iso_8601, time.gmtime())
