'''
HTML class.
'''

class HTML:
	'''
	A single HTML template.
	'''

	def __init__(self, *body):
		'''
		Initialise a new HTML object.
		'''

		self.body = ' '.join(body).strip()

	def __call__(self, func):
		'''
		Decorate a function to return a rendered template if the function
		returns a dictionary.
		'''

		def deco(*args, **kwargs):
			data = func(*args, **kwargs)
			return self.render(**data) if isinstance(data, dict) else data
		return deco

	def render(self, **kwargs):
		'''
		Return the template rendered with arguments.
		'''

		return self.body.format(**kwargs)
